@@ -1,15 +1,14 @@
 package Tk::Event;
 use vars qw($VERSION $XS_VERSION @EXPORT_OK);
 END { CleanupGlue() }
-$VERSION = '4.008'; # $Id: //depot/Tkutf8/Event/Event.pm#9 $
-$XS_VERSION = '804.024';
-require DynaLoader;
-use base  qw(Exporter DynaLoader);
+$VERSION = '4.016'; # was: sprintf '4.%03d', q$Revision: #15 $ =~ /\D(\d+)\s*$/;
+$XS_VERSION = '804.027_500';
+$XS_VERSION = eval $XS_VERSION;
+use base  qw(Exporter);
+use XSLoader;
 @EXPORT_OK = qw($XS_VERSION DONT_WAIT WINDOW_EVENTS  FILE_EVENTS
                 TIMER_EVENTS IDLE_EVENTS ALL_EVENTS);
-
-
-bootstrap Tk::Event;
+XSLoader::load 'Tk::Event',$XS_VERSION;
 require   Tk::Event::IO;
 1;
 __END__
