@@ -9,7 +9,7 @@ use strict;
 use Carp;
 
 use vars qw($VERSION);
-$VERSION = '4.006'; # $Id: //depot/Tkutf8/Tixish/DialogBox.pm#6 $
+$VERSION = '4.014'; # was: sprintf '4.%03d', q$Revision: #13 $ =~ /\D(\d+)\s*$/;
 
 use base  qw(Tk::Toplevel);
 
@@ -23,14 +23,18 @@ sub Populate {
     $buttons = ['OK'] unless defined $buttons;
     my $default_button = delete $args->{'-default_button'};
     $default_button = $buttons->[0] unless defined $default_button;
+    my $cancel_button = delete $args->{'-cancel_button'};
+    if (!$cancel_button && @$buttons == 1) {
+	$cancel_button = $buttons->[0];
+    }
 
     $cw->{'selected_button'} = '';
     $cw->transient($cw->Parent->toplevel);
     $cw->withdraw;
-    if (@$buttons == 1) {
-	$cw->protocol('WM_DELETE_WINDOW' => sub { $cw->{'default_button'}->invoke });
+    if ($cancel_button) {
+	$cw->protocol('WM_DELETE_WINDOW' => sub { $cw->{'cancel_button'}->invoke });
     } else {
-	$cw->protocol('WM_DELETE_WINDOW' => sub {});
+	$cw->protocol('WM_DELETE_WINDOW' => sub { $cw->{'selected_button'} = undef });
     }
 
     # create the two frames
@@ -53,22 +57,30 @@ sub Populate {
           $b->configure(-width => 10, -pady => 0);
          }
 	if ($bl eq $default_button) {
-            if ($Tk::platform eq 'MSWin32') {
-                $b->pack(-side => 'left', -expand => 1,  -padx => 1, -pady => 1);
-            } else {
-	        my $db = $bot->Frame(-relief => 'sunken', -bd => 1);
-	        $b->raise($db);
-	        $b->pack(-in => $db, -padx => '2', -pady => '2');
-	        $db->pack(-side => 'left', -expand => 1, -padx => 1, -pady => 1);
-            }
+	    my $db = $bot->Frame(-relief => 'sunken', -bd => 1);
+	    $b->raise($db);
+	    $b->pack(-in => $db, -padx => '2', -pady => '2');
+	    $db->pack(-side => 'left', -expand => 1, -padx => 1, -pady => 1);
 	    $cw->{'default_button'} = $b;
+	    $cw->bind('<Return>' => [ $b, 'Invoke']);
 	} else {
 	    $b->pack(-side => 'left', -expand => 1,  -padx => 1, -pady => 1);
 	}
+	if (defined $cancel_button && $bl eq $cancel_button) {
+	    $cw->{'cancel_button'} = $b;
+	}
+    }
+    if (defined $default_button && !$cw->{'default_button'}) {
+	warn "Default button `$default_button' does not exist.\n";
+    }
+    if (defined $cancel_button && !$cw->{'cancel_button'}) {
+	warn "Cancel button `$cancel_button' does not exist.\n";
     }
     $cw->ConfigSpecs(-command    => ['CALLBACK', undef, undef, undef ],
                      -foreground => ['DESCENDANTS', 'foreground','Foreground', 'black'],
                      -background => ['DESCENDANTS', 'background','Background',  undef],
+		     -focus	 => ['PASSIVE', undef, undef, undef],
+		     -showcommand => ['CALLBACK', undef, undef, undef],
                     );
     $cw->Delegates('Construct',$top);
 }
@@ -83,20 +95,24 @@ sub add {
 sub Wait
 {
  my $cw = shift;
+ $cw->Callback(-showcommand => $cw);
  $cw->waitVariable(\$cw->{'selected_button'});
- $cw->grabRelease;
- $cw->withdraw;
+ $cw->grabRelease if Tk::Exists($cw);
+ $cw->withdraw if Tk::Exists($cw);
  $cw->Callback(-command => $cw->{'selected_button'});
 }
 
 sub Show {
-    my ($cw, $grab) = @_;
+
     croak 'DialogBox: "Show" method requires at least 1 argument'
 	if scalar @_ < 1;
+    my $cw = shift;
+    my ($grab) = @_;
     my $old_focus = $cw->focusSave;
     my $old_grab = $cw->grabSave;
 
-    $cw->Popup();
+    shift if defined $grab && length $grab && ($grab =~ /global/);
+    $cw->Popup(@_);
 
     Tk::catch {
     if (defined $grab && length $grab && ($grab =~ /global/)) {
@@ -105,7 +121,9 @@ sub Show {
 	$cw->grab;
     }
     };
-    if (defined $cw->{'default_button'}) {
+    if (my $focusw = $cw->cget(-focus)) {
+	$focusw->focus;
+    } elsif (defined $cw->{'default_button'}) {
 	$cw->{'default_button'}->focus;
     } else {
 	$cw->focus;
@@ -116,4 +134,11 @@ sub Show {
     return $cw->{'selected_button'};
 }
 
+sub Exit
+{
+ my $cw = shift;
+ #kill the dialogbox, by faking a 'DONE'
+ $cw->{'selected_button'} = $cw->{'default_button'}->cget(-text);
+}
+
 1;
