@@ -9,7 +9,8 @@ package Tk::NoteBook;
 
 use vars qw($VERSION);
 
-$VERSION = '4.004'; # $Id: //depot/Tkutf8/Tixish/NoteBook.pm#4 $
+#$VERSION = sprintf '4.%03d', q$Revision: #9 $ =~ /\D(\d+)\s*$/;
+$VERSION = '4.010';
 require Tk::NBFrame;
 
 use base  qw(Tk::Derived Tk::NBFrame);
@@ -50,11 +51,12 @@ sub Populate
  my ($w, $args) = @_;
 
  $w->SUPER::Populate($args);
- $w->{'pad-x1'} = 0;
- $w->{'pad-x2'} = 0;
- $w->{'pad-y1'} = 0;
- $w->{'pad-y2'} = 0;
+ $w->{'pad-x1'} = undef;
+ $w->{'pad-x2'} = undef;
+ $w->{'pad-y1'} = undef;
+ $w->{'pad-y2'} = undef;
 
+ $w->{'windows'} = [];
  $w->{'nWindows'} = 0;
  $w->{'minH'} = 1;
  $w->{'minW'} = 1;
@@ -63,9 +65,9 @@ sub Populate
  $w->{'resize'} = 0;
 
  $w->ConfigSpecs(-ipadx => ['PASSIVE', 'ipadX', 'Pad', 0],
-                 -ipady => ['PASSIVE', 'ipadY', 'Pad', 0],
-                 -takefocus => ['SELF', 'takeFocus', 'TakeFocus', 0],
-                 -dynamicgeometry => ['PASSIVE', 'dynamicGeometry', 'DynamicGeometry', 0]);
+		 -ipady => ['PASSIVE', 'ipadY', 'Pad', 0],
+		 -takefocus => ['SELF', 'takeFocus', 'TakeFocus', 0],
+		 -dynamicgeometry => ['PASSIVE', 'dynamicGeometry', 'DynamicGeometry', 0]);
 
  # SetBindings
  $w->bind('<Configure>','MasterGeomProc');
@@ -161,6 +163,10 @@ sub raise
      my $myW = $w->Width;
      my $myH = $w->Height;
 
+     if (!defined $w->{'pad-x1'}) {
+	 $w->InitTabSize;
+     }
+
      my $cW = $myW - $w->{'pad-x1'} - $w->{'pad-x2'} - 2 * (defined $w->{-ipadx} ? $w->{-ipadx} : 0);
      my $cH = $myH - $w->{'pad-y1'} - $w->{'pad-y2'} - 2 * (defined $w->{-ipady} ? $w->{-ipady} : 0);
      my $cX = $w->{'pad-x1'} + (defined $w->{-ipadx} ? $w->{-ipadx} : 0);
@@ -175,9 +181,9 @@ sub raise
      if ((not defined $oldtop) || ($oldtop ne $child))
       {
        if (defined $childw->{-raisecmd})
-        {
-         $childw->{-raisecmd}->Call($childw);
-        }
+	{
+	 $childw->{-raisecmd}->Call($childw);
+	}
       }
     }
   }
@@ -226,7 +232,7 @@ sub delete
   {
    $childw->bind('<Destroy>', undef);
    $childw->destroy;
-   @{$w->{'windows'}} = grep($_ !~ /$child/, @{$w->{'windows'}});
+   @{$w->{'windows'}} = grep($_ ne $child, @{$w->{'windows'}});
    $w->{'nWindows'}--;
    $w->SUPER::delete($child);
    # see if the child to be deleted was the top child
@@ -259,12 +265,12 @@ sub MouseDown {
 sub MouseUp {
     my ($w, $x, $y) = @_;
     my $name = $w->identify($x, $y);
-    if ((defined $name) &&
-        ($name eq $w->{'down'}) &&
-        ($w->pagecget($name, -state) eq 'normal')) {
-        $w->raise($name);
+    if ((defined $name) && (defined $w->{'down'}) &&
+	($name eq $w->{'down'}) &&
+	($w->pagecget($name, -state) eq 'normal')) {
+	$w->raise($name);
     } else {
-        $w->focus($name);
+	$w->focus($name);
     }
 }
 
@@ -273,11 +279,11 @@ sub FocusNext {
     my $name;
 
     if (not defined $w->info('focus')) {
-        $name = $w->info('active');
-        $w->focus($name);
+	$name = $w->info('active');
+	$w->focus($name);
     } else {
-        $name = $w->info('focus' . $dir);
-        $w->focus($name);
+	$name = $w->info('focus' . $dir);
+	$w->focus($name);
     }
 }
 
@@ -286,10 +292,10 @@ sub SetFocusByKey {
 
     my $name = $w->info('focus');
     if (defined $name) {
-        if ($w->pagecget($name, -state) eq 'normal') {
-            $w->raise($name);
-            $w->activate($name);
-        }
+	if ($w->pagecget($name, -state) eq 'normal') {
+	    $w->raise($name);
+	    $w->activate($name);
+	}
     }
 }
 
@@ -298,13 +304,13 @@ sub NoteBookFind {
 
     my $page;
     foreach $page (@{$w->{'windows'}}) {
-        my $i = $w->pagecget($page, -underline);
-        my $c = substr($page, $i, 1);
-        if ($char =~ /$c/) {
-            if ($w->pagecget($page, -state) ne 'disabled') {
-                return $page;
-            }
-        }
+	my $i = $w->pagecget($page, -underline);
+	my $c = substr($page, $i, 1);
+	if ($char =~ /$c/) {
+	    if ($w->pagecget($page, -state) ne 'disabled') {
+		return $page;
+	    }
+	}
     }
     return undef;
 }
@@ -316,16 +322,16 @@ sub FindMenu {
 
     my $page;
     foreach $page (@{$w->{'windows'}}) {
-        my $i = $w->pagecget($page, -underline);
-        my $l = $w->pagecget($page, -label);
-        next if (not defined $l);
-        my $c = substr($l, $i, 1);
-        if ($char =~ /$c/i) {
-            if ($w->pagecget($page, -state) ne 'disabled') {
-                $w->raise($page);
-                return $w;
-            }
-        }
+	my $i = $w->pagecget($page, -underline);
+	my $l = $w->pagecget($page, -label);
+	next if (not defined $l);
+	my $c = substr($l, $i, 1);
+	if ($char =~ /$c/i) {
+	    if ($w->pagecget($page, -state) ne 'disabled') {
+		$w->raise($page);
+		return $w;
+	    }
+	}
     }
     return undef;
 }
@@ -352,7 +358,6 @@ sub SlaveGeometryRequest
 
 sub LostSlave {
     my ($w, $s) = @_;
-    print "Loosing $s\n";
     $s->UnmapWindow;
 }
 
@@ -378,13 +383,7 @@ sub Resize {
 
     return unless Tk::Exists($w) && $w->{'nWindows'} && $w->{'resize'};
 
-    my ($tW, $tH) = $w->geometryinfo;
-    $w->{'pad-x1'} = 2;
-    $w->{'pad-x2'} = 2;
-    $w->{'pad-y1'} = $tH + (defined $w->{'-ipadx'} ? $w->{'-ipadx'} : 0) + 1;
-    $w->{'pad-y2'} = 2;
-    $w->{'minW'} = $tW;
-    $w->{'minH'} = $tH;
+    $w->InitTabSize;
 
     $w->{'resize'} = 0;
     my $reqW = $w->{-width} || 0;
@@ -392,52 +391,62 @@ sub Resize {
 
     if ($reqW * $reqH == 0)
      {
-        if ((not defined $w->{-dynamicgeometry}) ||
-            ($w->{-dynamicgeometry} == 0)) {
-            $reqW = 1;
-            $reqH = 1;
+	if ((not defined $w->{-dynamicgeometry}) ||
+	    ($w->{-dynamicgeometry} == 0)) {
+	    $reqW = 1;
+	    $reqH = 1;
 
-            my $childw;
-            foreach $childw ($w->page_widget)
-             {
-                my $cW = $childw->ReqWidth;
-                my $cH = $childw->ReqHeight;
-                $reqW = $cW if ($reqW < $cW);
-                $reqH = $cH if ($reqH < $cH);
-            }
-        } else {
-            if (defined $w->{'topchild'}) {
-                my $topw = $w->page_widget($w->{'topchild'});
-                $reqW = $topw->ReqWidth;
-                $reqH = $topw->ReqHeight;
-            } else {
-                $reqW = 1;
-                $reqH = 1;
-            }
-        }
-        $reqW += $w->{'pad-x1'} + $w->{'pad-x2'} + 2 * (defined $w->{-ipadx} ? $w->{-ipadx} : 0);
-        $reqH += $w->{'pad-y1'} + $w->{'pad-y2'} + 2 * (defined $w->{-ipady} ? $w->{-ipady} : 0);
-        $reqW = ($reqW > $w->{'minW'}) ? $reqW : $w->{'minW'};
-        $reqH = ($reqH > $w->{'minH'}) ? $reqH : $w->{'minH'};
+	    my $childw;
+	    foreach $childw ($w->page_widget)
+	     {
+		my $cW = $childw->ReqWidth;
+		my $cH = $childw->ReqHeight;
+		$reqW = $cW if ($reqW < $cW);
+		$reqH = $cH if ($reqH < $cH);
+	    }
+	} else {
+	    if (defined $w->{'topchild'}) {
+		my $topw = $w->page_widget($w->{'topchild'});
+		$reqW = $topw->ReqWidth;
+		$reqH = $topw->ReqHeight;
+	    } else {
+		$reqW = 1;
+		$reqH = 1;
+	    }
+	}
+	$reqW += $w->{'pad-x1'} + $w->{'pad-x2'} + 2 * (defined $w->{-ipadx} ? $w->{-ipadx} : 0);
+	$reqH += $w->{'pad-y1'} + $w->{'pad-y2'} + 2 * (defined $w->{-ipady} ? $w->{-ipady} : 0);
+	$reqW = ($reqW > $w->{'minW'}) ? $reqW : $w->{'minW'};
+	$reqH = ($reqH > $w->{'minH'}) ? $reqH : $w->{'minH'};
     }
     if (($w->ReqWidth != $reqW) ||
-        ($w->ReqHeight != $reqH)) {
-        $w->{'counter'} = 0 if (not defined $w->{'counter'});
-        if ($w->{'counter'} < 50) {
-            $w->{'counter'}++;
-            $w->GeometryRequest($reqW, $reqH);
-            $w->afterIdle([$w,'Resize']);
-            $w->{'resize'} = 1;
-            return;
-        }
+	($w->ReqHeight != $reqH)) {
+	$w->{'counter'} = 0 if (not defined $w->{'counter'});
+	if ($w->{'counter'} < 50) {
+	    $w->{'counter'}++;
+	    $w->GeometryRequest($reqW, $reqH);
+	    $w->afterIdle([$w,'Resize']);
+	    $w->{'resize'} = 1;
+	    return;
+	}
     }
     $w->{'counter'} = 0;
     $w->raise($w->{'topchild'} || ${$w->{'windows'}}[0]);
     $w->{'resize'} = 0;
 }
 
+sub InitTabSize {
+    my ($w) = @_;
+    my ($tW, $tH) = $w->geometryinfo;
+    $w->{'pad-x1'} = 2;
+    $w->{'pad-x2'} = 2;
+    $w->{'pad-y1'} = $tH + (defined $w->{'-ipadx'} ? $w->{'-ipadx'} : 0) + 1;
+    $w->{'pad-y2'} = 2;
+    $w->{'minW'} = $tW;
+    $w->{'minH'} = $tH;
+}
+
 1;
 
 __END__
 
-=cut
