@@ -1,7 +1,7 @@
 #
 # Copyright (c) 1992-1994 The Regents of the University of California.
 # Copyright (c) 1994 Sun Microsystems, Inc.
-# Copyright (c) 1995-2000 Nick Ing-Simmons. All rights reserved.
+# Copyright (c) 1995-2004 Nick Ing-Simmons. All rights reserved.
 # This program is free software; you can redistribute it and/or
 
 # modify it under the same terms as Perl itself, subject
@@ -13,12 +13,14 @@ require 5.007;
 use     Tk::Event ();
 use     AutoLoader qw(AUTOLOAD);
 use     DynaLoader;
+use     Cwd();
 use base qw(Exporter DynaLoader);
 
 *fileevent = \&Tk::Event::IO::fileevent;
 
 use Encode;
-$Tk::encode_fallback = Encode::FB_QUIET();
+$Tk::encodeStopOnError = Encode::FB_QUIET();
+$Tk::encodeFallback    = Encode::FB_PERLQQ(); # Encode::FB_DEFAULT();
 
 our %font_encoding = ('jis0208' => 'jis0208-raw',
                       'jis0212' => 'jis0212-raw',
@@ -57,17 +59,22 @@ $Tk::tearoff = 1 if ($Tk::platform eq 'unix');
                );
 
 use strict;
-
 use Carp;
 
+# Record author's perforce depot record
+#$Tk::CHANGE      = q$Change: 3279 $;
+$Tk::CHANGE      = 'sfsvn-' . q$Change: 27 $;
+
 # $tk_version and $tk_patchLevel are reset by pTk when a mainwindow
 # is created, $VERSION is checked by bootstrap
 $Tk::version     = '8.4';
 $Tk::patchLevel  = '8.4';
-$Tk::VERSION     = '804.024';
+$Tk::VERSION     = '804.027_500';
+$Tk::VERSION     = eval $Tk::VERSION;
 $Tk::XS_VERSION  = $Tk::VERSION;
 $Tk::strictMotif = 0;
 
+
 {($Tk::library) = __FILE__ =~ /^(.*)\.pm$/;}
 $Tk::library = Tk->findINC('.') unless (defined($Tk::library) && -d $Tk::library);
 
@@ -88,6 +95,10 @@ use Tk::Submethods ('option'    =>  [qw(add get clear readfile)],
                     'clipboard' =>  [qw(clear append)]
                    );
 
+#
+# Next few routines are here as perl code as doing caller()
+# in XS code is very complicated - so instead C code calls BackTrace
+#
 sub _backTrace
 {
  my $w = shift;
@@ -128,7 +139,9 @@ sub __DIE__
  my $mess = shift;
  my $w = $Tk::widget;
  # Note that if a __DIE__ handler returns it re-dies up the chain.
- return unless defined $w;
+ return unless defined($w) && Exists($w);
+ # This special message is for exit() as an exception see pTkCallback.c
+ return if $mess =~/^_TK_EXIT_\(\d+\)/;
  return if $w->_backTrace;
  # Not in an eval - should not happen
 }
@@ -147,18 +160,15 @@ sub NoOp  { }
 
 sub Ev
 {
- my @args = @_;
- my $obj;
- if (@args == 1)
+ if (@_ == 1)
   {
-   my $arg = pop(@args);
-   $obj = (ref $arg) ? $arg : \$arg;
+   my $arg = $_[0];
+   return bless (((ref $arg) ? $arg : \$arg), 'Tk::Ev');
   }
  else
   {
-   $obj = \@args;
+   return bless [@_],'Tk::Ev';
   }
- return bless $obj,'Tk::Ev';
 }
 
 sub InitClass
@@ -245,6 +255,9 @@ my %dialog = ( tk_chooseColor => 'ColorDialog',
                tk_getOpenFile => 'FDialog',
                tk_getSaveFile => 'FDialog',
                tk_chooseDirectory => 'FDialog'
+# Slaven claims NI-S's version above does not work
+# and provides this
+#              tk_chooseDirectory => 'DirDialog'
              );
 
 foreach my $dialog (keys %dialog)
@@ -275,7 +288,7 @@ sub MessageBox {
                       split(/(abort|retry|ignore|yes|no|cancel|ok)/,
                             lc($type))));
 	$args->{-buttons} = [@buttons];
-	$args->{-default_button} = delete $args->{-default} if
+	$args->{-default_button} = ucfirst(delete $args->{-default}) if
 	    defined $args->{-default};
 	if (not defined $args->{-default_button} and scalar(@buttons) == 1) {
 	   $args->{-default_button} = $buttons[0];
@@ -290,8 +303,18 @@ sub MessageBox {
 sub messageBox
 {
  my ($widget,%args) = @_;
- $args{'-type'} = (exists $args{'-type'}) ? lc($args{'-type'}) : 'ok';
- tk_messageBox(-parent => $widget, %args);
+ # remove in a later version:
+ if (exists $args{'-text'})
+  {
+   warn "The -text option is deprecated. Please use -message instead";
+   if (!exists $args{'-message'})
+    {
+     $args{'-message'} = delete $args{'-text'};
+    }
+  }
+ $args{'-type'}    = (exists $args{'-type'})    ? lc($args{'-type'}) : 'ok';
+ $args{'-default'} = lc($args{'-default'}) if (exists $args{'-default'});
+ ucfirst tk_messageBox(-parent => $widget, %args);
 }
 
 sub getOpenFile
@@ -309,6 +332,11 @@ sub chooseColor
  tk_chooseColor(-parent => shift,@_);
 }
 
+sub chooseDirectory
+{
+ tk_chooseDirectory(-parent => shift,@_);
+}
+
 sub DialogWrapper
 {
  my ($method,$kind,%args) = @_;
@@ -324,7 +352,7 @@ sub DialogWrapper
    $w->withdraw;
    $created = 1;
   }
- my $mw = $w->MainWindow;
+ my $mw = $w->toplevel;
  my $fs = $mw->{$kind};
  unless (defined $fs)
   {
@@ -353,11 +381,23 @@ sub FDialog
   {
    push @_, -type => 'save';
   }
+ elsif ($cmd =~ /Directory/)
+  {
+   push @_, -type => 'dir';
+  }
  DialogWrapper('FBox', $cmd, @_);
 }
 
+sub DirDialog
+{
+ require Tk::DirTree;
+ DialogWrapper('DirTreeDialog',@_);
+}
+
 *MotifFDialog = \&FDialog;
 
+*CORE::GLOBAL::exit = \&exit;
+
 sub MainLoop
 {
  unless ($inMainLoop)
@@ -386,7 +426,7 @@ sub TranslateFileName
  local $_ = shift;
  unless (defined $Home)
   {
-   $Home = $ENV{'HOME'} || ($ENV{'HOMEDRIVE'}.$ENV{'HOMEPATH'});
+   $Home = $ENV{'HOME'} || (defined $ENV{'HOMEDRIVE'} && defined $ENV{'HOMEPATH'} ? $ENV{'HOMEDRIVE'}.$ENV{'HOMEPATH'} : "");
    $Home =~ s#\\#/#g;
    $Home .= '/' unless $Home =~ m#/$#;
   }
@@ -415,23 +455,21 @@ sub idletasks
 
 sub backtrace
 {
- my ($self,$msg) = @_;
- my $i = 1;
- my ($pack,$file,$line,$sub) = caller($i++);
+ my ($self,$msg,$i) = @_;
+ $i = 1 if @_ < 3;
  while (1)
   {
-   my $loc = "at $file line $line";
-   ($pack,$file,$line,$sub) = caller($i++);
+   my ($pack,$file,$line,$sub) = caller($i++);
    last unless defined($sub);
-   $msg .= " $sub $loc\n";
+   $msg .= "\n $sub at $file line $line";
   }
- return $msg;
+ return "$msg\n";
 }
 
 sub die_with_trace
 {
  my ($self,$msg) = @_;
- die $self->backtrace($msg);
+ die $self->backtrace($msg,1);
 }
 
 
@@ -598,8 +636,11 @@ sub FocusOK
   {
    return 0 if ($value eq '0');
    return $w->viewable if ($value eq '1');
-   $value = $w->$value();
-   return $value if (defined $value);
+   if ($value)
+    {
+     $value = $w->$value();
+     return $value if (defined $value);
+    }
   }
  if (!$w->viewable)
   {
@@ -628,6 +669,7 @@ sub FocusOK
 sub EnterFocus
 {
  my $w  = shift;
+ return unless $w;
  my $Ev = $w->XEvent;
  my $d  = $Ev->d;
  $w->Tk::focus() if ($d eq 'NotifyAncestor' ||  $d eq 'NotifyNonlinear' ||  $d eq 'NotifyInferior');
