@@ -1,7 +1,7 @@
 package Tk::FileSelect;
 
 use vars qw($VERSION @EXPORT_OK);
-$VERSION = '4.010'; # $Id: //depot/Tkutf8/Tk/FileSelect.pm#10 $
+$VERSION = '4.016'; # was: sprintf '4.%03d', q$Revision: #15 $ =~ /\D(\d+)\s*$/;
 @EXPORT_OK = qw(glob_to_re);
 
 use Tk qw(Ev);
@@ -50,8 +50,17 @@ sub import {
     if (defined $_[1] and $_[1] eq 'as_default') {
 	local $^W = 0;
 	package Tk;
-	*FDialog      = \&Tk::FileSelect::FDialog;
-	*MotifFDialog = \&Tk::FileSelect::FDialog;
+	if ($Tk::VERSION < 804) {
+	    *FDialog      = \&Tk::FileSelect::FDialog;
+	    *MotifFDialog = \&Tk::FileSelect::FDialog;
+	} else {
+	    *tk_getOpenFile = sub {
+		Tk::FileSelect::FDialog("tk_getOpenFile", @_);
+	    };
+	    *tk_getSaveFile = sub {
+		Tk::FileSelect::FDialog("tk_getSaveFile", @_);
+	    };
+	}
     }
 }
 
@@ -177,6 +186,7 @@ sub Populate {
     );
     $e->pack(-side => 'bottom', -expand => 0, -fill => 'x');
     $e->bind('<Return>' => [$w => 'validateFile', Ev(['get'])]);
+    $e->bind('<FocusIn>' => [$w => 'SelectionClear']);
 
     # Create directory scrollbox, place at the left-middle.
     my $b = $w->Component(
@@ -564,5 +574,3 @@ sub FDialog
 
 __END__
 
-=cut
-
