@@ -1,9 +1,9 @@
-# Copyright (c) 1995-2000 Nick Ing-Simmons. All rights reserved.
+# Copyright (c) 1995-2004 Nick Ing-Simmons. All rights reserved.
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 package Tk::Widget;
 use vars qw($VERSION @DefaultMenuLabels);
-$VERSION = '4.017'; # $Id: //depot/Tkutf8/Tk/Widget.pm#17 $
+$VERSION = '4.031'; # was: sprintf '4.%03d', q$Revision: #30 $ =~ /\D(\d+)\s*$/;
 
 require Tk;
 use AutoLoader;
@@ -12,20 +12,22 @@ use Carp;
 use base qw(DynaLoader Tk);
 
 # stubs for 'autoloaded' widget classes
-
 sub Button;
 sub Canvas;
 sub Checkbutton;
 sub Entry;
 sub Frame;
 sub Label;
+sub Labelframe;
 sub Listbox;
 sub Menu;
 sub Menubutton;
 sub Message;
+sub Panedwindow;
+sub Radiobutton;
 sub Scale;
 sub Scrollbar;
-sub Radiobutton;
+sub Spinbox;
 sub Text;
 sub Toplevel;
 
@@ -66,7 +68,7 @@ use Tk::Submethods( 'grab' =>  [qw(current status release -global)],
                     'event' => [qw(add delete generate info)],
                     'place' => [qw(configure forget info slaves)],
                     'wm'    => [qw(capture release)],
-                    'font'  => [qw(actual configure create delete families measure metrics names)]
+                    'font'  => [qw(actual configure create delete families measure metrics names subfonts)]
                   );
 
 BEGIN  {
@@ -87,7 +89,7 @@ Direct Tk::Submethods (
                visualsavailable  vrootheight viewable vrootwidth vrootx vrooty
                width x y toplevel children pixels pointerx pointery pointerxy
                server fpixels rgb )],
-   'tk'   => [qw(appname scaling)]);
+   'tk'   => [qw(appname caret scaling useinputmethods windowingsystem)]);
 
 
 sub DESTROY
@@ -130,7 +132,7 @@ sub CreateArgs
  my @result = ();
  my $class = delete $args->{'-class'};
  ($class) = $package =~ /([A-Z][A-Z0-9_]*)$/i unless (defined $class);
- push(@result, '-class' => "\u$class") if (defined $class);
+ @result = (-class => "\u$class") if (defined $class);
  foreach my $opt ($package->CreateOptions)
   {
    push(@result, $opt => delete $args->{$opt}) if exists $args->{$opt};
@@ -270,7 +272,7 @@ sub _AutoloadTkWidget
  return $what;
 }
 
-require UNIVERSAL;
+# require UNIVERSAL; don't load .pm use XS code from perl core though
 
 sub AUTOLOAD
 {
@@ -482,12 +484,6 @@ sub focusSave
 # but does auto-cancel when widget is deleted.
 require Tk::After;
 
-sub afterIdle
-{
- my $w = shift;
- return Tk::After->new($w,'idle','once',@_);
-}
-
 sub afterCancel
 {
  my ($w,$what) = @_;
@@ -499,6 +495,21 @@ sub afterCancel
   }
 }
 
+sub afterIdle
+{
+ my $w = shift;
+ return Tk::After->new($w,'idle','once',@_);
+}
+
+sub afterInfo {
+    my ($w, $id) = @_;
+    if (defined $id) {
+	return ($id->[4], $id->[2], $id->[3]);
+    } else {
+	return sort( keys %{$w->{_After_}} );
+    }
+}
+
 sub after
 {
  my $w = shift;
@@ -825,8 +836,24 @@ sub BusyRecurse
 
 sub Busy
 {
- my ($w,%args) = @_;
+ my ($w,@args) = @_;
  return unless $w->viewable;
+ my($sub, %args);
+ for(my $i=0; $i<=$#args; $i++)
+  {
+   if (ref $args[$i] eq 'CODE')
+    {
+     if (defined $sub)
+      {
+       croak "Multiple code definitions not allowed in Tk::Widget::Busy";
+      }
+     $sub = $args[$i];
+    }
+   else
+    {
+     $args{$args[$i]} = $args[$i+1]; $i++;
+    }
+  }
  my $cursor  = delete $args{'-cursor'};
  my $recurse = delete $args{'-recurse'};
  $cursor  = 'watch' unless defined $cursor;
@@ -861,8 +888,18 @@ sub Busy
    $g->grabRelease;
   }
  $w->update;
- eval {local $SIG{'__DIE__'};  $w->grab };
- $w->update;
+ if (Tk::Exists($w))
+  {
+   eval {local $SIG{'__DIE__'};  $w->grab };
+   $w->update;
+  }
+ if ($sub && Tk::Exists($w))
+  {
+   eval { $sub->() };
+   my $err = $@;
+   $w->Unbusy(-recurse => $recurse);
+   die $err if $err;
+  }
 }
 
 sub _busy
@@ -960,6 +997,8 @@ sub XMouseWheelBind
  # <4> and <5> are how mousewheel looks on X
  $mw->bind($class,'<Shift-4>',      ['xview','scroll',-1,'units']);
  $mw->bind($class,'<Shift-5>',      ['xview','scroll',1,'units']);
+ $mw->bind($class,'<Button-6>',     ['xview','scroll',-1,'units']);
+ $mw->bind($class,'<Button-7>',     ['xview','scroll',1,'units']);
 }
 
 sub YMouseWheelBind
@@ -987,6 +1026,34 @@ sub XYscrollBind
  # <4> and <5> are how mousewheel looks on X
 }
 
+sub MouseWheelBind
+{
+ my($mw,$class) = @_;
+
+ # The MouseWheel will typically only fire on Windows. However, one
+ # could use the "event generate" command to produce MouseWheel
+ # events on other platforms.
+
+ $mw->Tk::bind($class, '<MouseWheel>',
+	       [ sub { $_[0]->yview('scroll',-($_[1]/120)*3,'units') }, Tk::Ev("D")]);
+
+ if ($Tk::platform eq 'unix')
+  {
+   # Support for mousewheels on Linux/Unix commonly comes through mapping
+   # the wheel to the extended buttons.  If you have a mousewheel, find
+   # Linux configuration info at:
+   #   http://www.inria.fr/koala/colas/mouse-wheel-scroll/
+   $mw->Tk::bind($class, '<4>',
+		 sub { $_[0]->yview('scroll', -3, 'units')
+			   unless $Tk::strictMotif;
+		   });
+   $mw->Tk::bind($class, '<5>',
+		 sub { $_[0]->yview('scroll', 3, 'units')
+			   unless $Tk::strictMotif;
+		   });
+  }
+}
+
 sub ScrlListbox
 {
  my $parent = shift;
@@ -1123,6 +1190,7 @@ sub form
 sub Scrolled
 {
  my ($parent,$kind,%args) = @_;
+ $kind = 'Pane' if $kind eq 'Frame';
  # Find args that are Frame create time args
  my @args = Tk::Frame->CreateArgs($parent,\%args);
  my $name = delete $args{'Name'};
@@ -1243,11 +1311,133 @@ sub BalloonInfo
   }
 }
 
+sub ConfigSpecs {
 
+    my $w = shift;
+
+    return map { ( $_->[0], [ $w, @$_[ 1 .. 4 ] ] ) } $w->configure;
+
+}
+
+*GetSelection =
+    ($Tk::platform eq 'unix'
+     ? sub
+        {
+         my $w = shift;
+         my $sel = @_ ? shift : "PRIMARY";
+         my $txt = eval { local $SIG{__DIE__};
+			  $w->SelectionGet(-selection => $sel, -type => "UTF8_STRING")
+  		        };
+         if ($@)
+	  {
+  	   $txt = eval { local $SIG{__DIE__};
+			 $w->SelectionGet(-selection => $sel)
+  		       };
+  	 if ($@)
+  	  {
+  	   die "could not find default selection";
+            }
+          }
+         $txt;
+        }
+     : sub
+        {
+	 my $w = shift;
+	 my $sel = @_ ? shift : "PRIMARY";
+	 my $txt = eval { local $SIG{__DIE__};
+			  $w->SelectionGet(-selection => $sel)
+		        };
+	 if ($@)
+	  {
+	   die "could not find default selection";
+          }
+	 $txt;
+        }
+    );
 
 1;
 __END__
 
+sub bindDump {
+
+    # Dump lots of good binding information.  This pretty-print subroutine
+    # is, essentially, the following code in disguise:
+    #
+    # print "Binding information for $w\n";
+    # foreach my $tag ($w->bindtags) {
+    #     printf "\n Binding tag '$tag' has these bindings:\n";
+    #     foreach my $binding ($w->bind($tag)) {
+    #         printf "  $binding\n";
+    #     }
+    # }
+
+    my ($w) = @_;
+
+    my (@bindtags) = $w->bindtags;
+    my $digits = length( scalar @bindtags );
+    my ($spc1, $spc2) = ($digits + 33, $digits + 35);
+    my $format1 = "%${digits}d.";
+    my $format2 = ' ' x ($digits + 2);
+    my $n = 0;
+
+    my @out;
+    push @out, sprintf( "\n## Binding information for '%s', %s ##", $w->PathName, $w );
+
+    foreach my $tag (@bindtags) {
+        my (@bindings) = $w->bind($tag);
+        $n++;                   # count this bindtag
+
+        if ($#bindings == -1) {
+            push @out, sprintf( "\n$format1 Binding tag '$tag' has no bindings.\n", $n );
+        } else {
+            push @out, sprintf( "\n$format1 Binding tag '$tag' has these bindings:\n", $n );
+
+            foreach my $binding ( @bindings ) {
+                my $callback = $w->bind($tag, $binding);
+                push @out, sprintf( "$format2%27s : %-40s\n", $binding, $callback );
+
+                if ($callback =~ /SCALAR/) {
+                    if (ref $$callback) {
+                        push @out, sprintf( "%s %s\n", ' ' x $spc1, $$callback );
+                    } else {
+                        push @out, sprintf( "%s '%s'\n", ' ' x $spc1, $$callback );
+                    }
+                } elsif ($callback =~ /ARRAY/) {
+                    if (ref $callback->[0]) {
+                        push @out, sprintf( "%s %s\n", ' ' x $spc1, $callback->[0], "\n" );
+                    } else {
+                        push @out, sprintf( "%s '%s'\n", ' ' x $spc1, $callback->[0], "\n" );
+                    }
+                    foreach my $arg (@$callback[1 .. $#{@$callback}]) {
+                        if (ref $arg) {
+                            push @out, sprintf( "%s %-40s", ' ' x $spc2, $arg );
+                        } else {
+                            push @out, sprintf( "%s '%s'", ' ' x $spc2, $arg );
+                        }
+			
+                        if (ref $arg eq 'Tk::Ev') {
+                            if ($arg =~ /SCALAR/) {
+                                push @out, sprintf( ": '$$arg'" );
+                            } else {
+                                push @out, sprintf( ": '%s'", join("' '", @$arg) );
+                            }
+                        }
+
+                        push @out, sprintf( "\n" );
+                    } # forend callback arguments
+                } # ifend callback
+
+            } # forend all bindings for one tag
+
+        } # ifend have bindings
+
+    } # forend all tags
+    push @out, sprintf( "\n" );
+    return @out;
+
+} # end bindDump
+
+
 sub ASkludge
 {
  my ($hash,$sense) = @_;
@@ -1322,5 +1512,185 @@ sub pathname
  return $x->PathName;
 }
 
+# ::tk::UnderlineAmpersand --
+# This procedure takes some text with ampersand and returns
+# text w/o ampersand and position of the ampersand.
+# Double ampersands are converted to single ones.
+# Position returned is -1 when there is no ampersand.
+#
+sub UnderlineAmpersand
+{
+ my (undef,$text) = @_;
+ if ($text =~ m{(?<!&)&(?!&)}g)
+  {
+   my $idx = pos $text;
+   $text =~ s{(?<!&)&(?!&)}{};
+   ($text, $idx);
+  }
+ else
+  {
+   ($text, -1);
+  }
+}
 
+# ::tk::SetAmpText -- 
+# Given widget path and text with "magic ampersands",
+# sets -text and -underline options for the widget
+#
+sub SetAmpText
+{
+ my ($w,$text) = @_;
+ my ($newtext,$under) =  $w->UnderlineAmpersand($text);
+ $w->configure(-text => $newtext, -underline => $under);
+}
 
+# ::tk::AmpWidget --
+# Creates new widget, turning -text option into -text and
+# -underline options, returned by ::tk::UnderlineAmpersand.
+#
+sub AmpWidget
+{
+ my ($w,$class,%args) = @_;
+ my @options;
+ while(my($opt,$val) = each %args)
+  {
+   if ($opt eq "-text")
+    {
+     my ($newtext,$under) = $w->UnderlineAmpersand($val);
+     push @options, -text => $newtext, -underline => $under;
+    }
+   else
+    {
+     push @options, $opt, $val;
+    }
+  }
+ my $result = $w->$class(@options);
+ if ($result->can('AmpWidgetPostHook'))
+  {
+   $result->AmpWidgetPostHook;
+  }
+ return $result;
+}
+
+# ::tk::FindAltKeyTarget --
+# search recursively through the hierarchy of visible widgets
+# to find button or label which has $char as underlined character
+#
+sub FindAltKeyTarget
+{
+ my ($w,$char) = @_;
+ $char = lc $char;
+ if ($w->isa('Tk::Button') || $w->isa('Tk::Label'))
+  {
+   if ($char eq lc substr($w->cget(-text), $w->cget(-underline), 1))
+    {
+     return $w;
+    }
+   else
+    {
+     return undef;
+    }
+  }
+ else
+  {
+   for my $cw ($w->gridSlaves, $w->packSlaves, $w->placeSlaves) # Cannot handle $w->formSlaves here?
+    {
+     my $target = $cw->FindAltKeyTarget($char);
+     return $target if ($target);
+    }
+  }
+ undef;
+}
+
+# ::tk::AltKeyInDialog --
+# <Alt-Key> event handler for standard dialogs. Sends <<AltUnderlined>>
+# to button or label which has appropriate underlined character
+#
+sub AltKeyInDialog
+{
+ my ($w, $key) = @_;
+ my $target = $w->FindAltKeyTarget($key);
+ return if !$target;
+ $target->eventGenerate('<<AltUnderlined>>');
+}
+
+# ::tk::SetFocusGrab --
+#   swap out current focus and grab temporarily (for dialogs)
+# Arguments:
+#   grab	new window to grab
+#   focus	window to give focus to
+# Results:
+#   Returns nothing
+#
+sub SetFocusGrab
+{
+ my ($grab,$focus) = @_;
+ my $index = "$grab,$focus";
+ $Tk::FocusGrab{$index} ||= [];
+ my $data = $Tk::FocusGrab{$index};
+ push @$data, $grab->focusCurrent;
+ my $oldGrab = $grab->grabCurrent;
+ push @$data, $oldGrab;
+ if (Tk::Exists($oldGrab))
+  {
+   push @$data, $oldGrab->grabStatus;
+  }
+ # The "grab" command will fail if another application
+ # already holds the grab.  So catch it.
+ Tk::catch { $grab->grab };
+ if (Tk::Exists($focus))
+  {
+   $focus->focus;
+  }
+}
+
+# ::tk::RestoreFocusGrab --
+#   restore old focus and grab (for dialogs)
+# Arguments:
+#   grab	window that had taken grab
+#   focus	window that had taken focus
+#   destroy	destroy|withdraw - how to handle the old grabbed window
+# Results:
+#   Returns nothing
+#
+sub RestoreFocusGrab
+{
+ my ($grab, $focus, $destroy) = @_;
+ $destroy = 'destroy' if !$destroy;
+ my $index = "$grab,$focus";
+ my ($oldFocus, $oldGrab, $oldStatus);
+ if (exists $Tk::FocusGrab{$index})
+  {
+   ($oldFocus, $oldGrab, $oldStatus) = $Tk::FocusGrab{$index};
+   delete $Tk::FocusGrab{$index};
+  }
+ else
+  {
+   $oldGrab = "";
+  }
+
+ Tk::catch { $oldFocus->focus };
+ if (Tk::Exists($grab))
+  {
+   $grab->grabRelease;
+   if ($destroy eq "withdraw")
+    {
+     $grab->withdraw;
+    }
+   else
+    {
+     $grab->destroy;
+    }
+  }
+ if (Tk::Exists($oldGrab) && $oldGrab->ismapped)
+  {
+   if ($oldStatus eq "global")
+    {
+     $oldGrab->grabGlobal;
+    }
+   else
+    {
+     $oldGrab->grab;
+    }
+  }
+}
